// This file is part of Projecteur - https://github.com/jahnf/projecteur
// - See LICENSE.md and README.md

#include "devicescan.h"

#include <QtCore>

#include <vector>

// Generated during CMake configuration time

namespace {
  // List of supported extra-devices
  const std::vector<SupportedDevice> supportedExtraDevices { // 
    {0x0c45, 0x8101, false, "AVATTO H100 / August WP200"}, // AVATTO H100 / August WP200
    {0x2312, 0x863d, false, "August LP315"}, // August LP315
    {0x2571, 0x4109, false, "AVATTO i10 Pro"}, // AVATTO i10 Pro
    {0x17ef, 0x60d9, false, "Lenovo ThinkPad X1 Presenter Mouse"}, // Lenovo ThinkPad X1 Presenter Mouse
    {0x17ef, 0x60db, true, "Lenovo ThinkPad X1 Presenter Mouse"}, // Lenovo ThinkPad X1 Presenter Mouse
    {0x69a7, 0x9803, false, "August LP310"}, // August LP310
    {0x3243, 0x0122, false, "Norwii Wireless Presenter"}, // Norwii Wireless Presenter
  };
} // end anonymous namespace

// Function declaration to check for extra devices, definition in generated source
bool isExtraDeviceSupported(quint16 vendorId, quint16 productId)
{
  const auto it = std::find_if(supportedExtraDevices.cbegin(), supportedExtraDevices.cend(),
  [vendorId, productId](const SupportedDevice& d) {
    return (vendorId == d.vendorId) && (productId == d.productId);
  });
  return it != supportedExtraDevices.cend();
};

QString getExtraDeviceName(quint16 vendorId, quint16 productId)
{
  const auto it = std::find_if(supportedExtraDevices.cbegin(), supportedExtraDevices.cend(),
  [vendorId, productId](const SupportedDevice& d) {
    return (vendorId == d.vendorId) && (productId == d.productId);
  });
  if (it != supportedExtraDevices.cend()) return it->name;
  return QString();
};
